@@ -26,22 +26,27 @@
 #'      \item{finalFile}{[NULL]  filename where to save \code{envT$theFinals}, only relevant for \code{tdm$fileMode==TRUE}}
 #'      \item{experFile}{[NULL] filename where to append \code{envT$theFinals}, only relevant for \code{tdm$fileMode==TRUE}  }
 #'      \item{theSpotPath}{[NA] use SPOT's package version}
-#'      \item{parallelCPUs}{[1] 1: sequential, >1: parallel with snowFall and this many CPUs}
-#'      \item{parallelFuncs}{[NULL] in case tdm$parallelCPUs>1: a string vector with functions which are sfExport'ed in addition
+#'      \item{parallelCPUs}{[1] 1: sequential, >1: parallel execution with this many CPUs (package parallel)  }
+#'      \item{parallelFuncs}{[NULL] in case tdm$parallelCPUs>1: a string vector with functions which are clusterExport'ed in addition
 #'                to tdm$mainFunc.  }
 #'      \item{path}{[getwd()] where to search .conf and .apd file}
 #'      \item{stratified}{[NULL] see \code{\link{tdmSplitTestData}}  }
 #'      \item{tdmPath}{[NULL] from where to source the R sources. If NULL load library TDMR instead.  }
 #'      \item{test2.string}{["default cutoff"] }
 #'      \item{optsVerbosity}{[0] the verbosity for the unbiased runs}
+#'      \item{withParams}{[TRUE] list the columns with tuned parameter in final results  }
 #'      \item{nrun}{[5] number of runs for unbiased runs}
-#'      \item{tstFrac}{[0.2] test set fraction for unbiased runs (only for umode="RSUB") }
 #'      \item{tstCol}{["TST"] opts$TST.COL for unbiased runs (only for umode="TST") }
 #'      \item{nfold}{[10] number of CV-folds for unbiased runs (only for umode="CV") }
-#'      \item{TST.trnFrac}{[NULL] train set fraction, copied to opts$TST.trnFrac if not NULL. }
-#'      \item{TST.valiFrac}{[NULL] validation set fraction, copied to opts$TST.valiFrac if not NULL. }
+#'      \item{TST.trnFrac}{[NULL] train set fraction (of all train-vali data),OVERWRITES opts$TST.trnFrac if not NULL. }
+#'      \item{TST.valiFrac}{[NULL] validation set fraction (of all train-vali data), OVERWRITES to opts$TST.valiFrac if not NULL. }
+#'      \item{TST.testFrac}{[0.2] test set fraction (of *all* data) for unbiased runs (only for umode="RSUB" or ="SP_T") }
 #'
 #' @note 
+#'      The settings \code{tdm$TST.trnFrac} and \code{tdm$TST.valiFrac} allow to set programmatically certain values for
+#'      \code{opts$TST.trnFrac} and \code{opts$TST.valiFrac} *after* \code{opts} has been read from APD file. So use 
+#'      \code{tdm$TST.trnFrac} and \code{tdm$TST.valiFrac} with CAUTION!
+#'
 #'      For \code{tdm$timeMode}, the 'user time' is the CPU time charged for the execution of user instructions of the calling process. 
 #'      The 'system time' is the CPU time charged for execution by the system on behalf of the calling process. 
 #'      The 'elapsed time' is the 'real' (wall-clock) time since the process was started.
@@ -67,6 +72,7 @@ tdmDefaultsFill <- function(tdm=NULL,mainFile=NULL) {
   if (is.null(tdm$timeMode)) tdm$timeMode <- 1;             # user time
   if (is.null(tdm$fileMode)) tdm$fileMode <- FALSE;         # 10/2012 /WK/ changed from prior 'TRUE'
   if (is.null(tdm$optsVerbosity)) tdm$optsVerbosity <- 0;   # the verbosity for the unbiased runs
+  if (is.null(tdm$withParams)) tdm$withParams <- TRUE;      # list the columns with tuned parameter in final results 
   if (is.null(tdm$theSpotPath)) tdm$theSpotPath <- NA;
   if (is.null(tdm$parallelCPUs)) tdm$parallelCPUs <- 1;
   if (is.null(tdm$path)) tdm$path <- paste(getwd(),"/",sep="");
@@ -79,7 +85,12 @@ tdmDefaultsFill <- function(tdm=NULL,mainFile=NULL) {
   # (cleaner code, less places where tdm values are set)
   if (is.null(tdm$test2.string)) tdm$test2.string="default cutoff";
   if (is.null(tdm$tstCol)) tdm$tstCol="TST";
-  if (is.null(tdm$tstFrac)) tdm$tstFrac=0.2;
+  if (!is.null(tdm$tstFrac)) {
+    # this section only as long as tdm$tstFrac is still def'd in some script_* or other files
+    cat("NOTE: Deprecated value tdm$tstFrac used. It will overwrite tdm$TST.testFrac.\n");
+    tdm$TST.testFrac = tdm$tstFrac;
+  }
+  if (is.null(tdm$TST.testFrac)) tdm$TST.testFrac=0.2;
   if (is.null(tdm$nfold)) tdm$nfold=10;
   if (is.null(tdm$nrun)) tdm$nrun=5;
 
