@@ -51,7 +51,8 @@
 #' @examples
 #'    \dontrun{ 
 #'      ##
-#'      ## Read previous tuning results 'envT' from demo02sonar/demoSonar.RData (relative to the TDMR package directory). 
+#'      ## Read previous tuning results 'envT' from demo02sonar/demoSonar.RData 
+#'      ## (relative to the TDMR package directory). 
 #'      ## Then, tdmPlotResMeta lets you explore interactively the RES data frame(s):
 #'      load(paste(.find.package("TDMR"), "demo02sonar","demoSonar.RData",sep="/"));
 #'      tdmPlotResMeta(envT);
@@ -318,6 +319,7 @@ testMeta <- function(envT) {
         sC$seq.predictionModel.func = "spotPredictMlegp" # "spotPredictForrester" # "spotPredictGausspr" #
         sC$seq.mlegp.min.nugget = 0.5;     # NEW, experimental!
 
+        sC$spot.fileMode=F;
         sC <- reportTestMeta(sC,newRes); 
         print(head(cbind(sC$newDes,sC$newY,sC$meta.Y),15));
         cat("sC$meta.newMAD = ",sC$meta.newMAD,"\n");
