@@ -3,15 +3,11 @@
 #
 #'     Perform unbiased runs with best-solution parameters.
 #'
-#'     Read the best solution of a parameter-tuning run (either from envT$bst or from file), 
+#'     Read the best solution of a parameter-tuning run from envT$bst, 
 #'     execute with these best parameters the function \code{tdm$mainFunc} (usually a classification or 
 #'     regression machine learning task), to see 
 #'     whether the result quality is reproducible on independent test data or on independently trained models.
 #'
-#'     If envT$bst or envT$res is NULL, try to read it from the file (the filename is
-#'     inferred envT$spotConfig. If this is NULL, it is constructed from confFile). 
-#'     We try to find the files for envT$bst or envT$res in dir envT$theTuner).
-#'
 #'   @param confFile    the .conf filename, e.g. "appAcid_02.conf"
 #'   @param envT        environment, from which we need the objects
 #'     \describe{
@@ -24,21 +20,23 @@
 #'   @param finals      [NULL] a one-row data frame to which new columns with final results are added
 #'   @param umode       [ "RSUB" (default) | "CV" | "TST" | "SP_T" ], how to divide in training and test data for the unbiased runs:
 #'     \describe{
-#'     \item{\code{"RSUB"}}{ random subsampling into (1-tdm$tstFrac)\% training and tdm$tstFrac\% test data}
+#'     \item{\code{"RSUB"}}{ random subsampling into (1-tdm$TST.testFrac)\% training and tdm$TST.testFrac\% test data}
 #'     \item{\code{"CV"}}{ cross validation (CV) with tdm$nrun folds}
 #'     \item{\code{"TST"}}{ all data in opts$filename are used for training, all data in opts$filetest for testing}
-#'     \item{\code{"SP_T"}}{ 'split_test': prior to tuning, the data set was split by random subsampling into test and training-vali data. 
-#'                  Tuning was done on training-vali data, now we use column "tdmSplit" to select the test data for unbiased evaluation.}
+#'     \item{\code{"SP_T"}}{ 'split_test': prior to tuning, the data set was split by random subsampling into tdm$TST.testFrac\% test and
+#'                  (1-tdm$TST.testFrac)\% training-vali data, tagged via column "tdmSplit". Tuning was done on training-vali data.
+#'                  Now we use column "tdmSplit" to select the test data for unbiased evaluation. Training during unbiased evaluation
+#'                  is done on a fraction tdm$TST.trnFrac of the training-vali data}
 #'     }
 #'   @param withParams  [FALSE] if =TRUE, add columns with best parameters to data frame finals
 #'                      (should be FALSE, if different runs have different parameters)
 #'   @param tdm         a list with TDM settings from which we use here the elements
 #'     \describe{
-#'     \item{mainFunction}{ the function to be called for unbiased evaluations}
+#'     \item{mainFunc}{ the function to be called for unbiased evaluations}
 #'     \item{mainFile}{ change to the directory of mainFile before starting mainFunc}
 #'     \item{nrun}{ [5] how often to call the unbiased evaluation}
 #'     \item{nfold}{ [10] how many folds in CV (only relevant for umode="CV") }
-#'     \item{tstFrac}{ [0.2] test set fraction (only relevant for umode="RSUB") }
+#'     \item{TST.testFrac}{ [0.2] test set fraction (only relevant for umode="RSUB" or ="SP_T") }
 #'     }
 #'     The defaults in '[...]' are set by  \code{\link{tdmDefaultsFill}}, if they are not defined on input.
 #'   @return \code{finals}     a one-row data frame with final results
@@ -50,12 +48,14 @@
 #'    \code{spotConfig$io.apdFileName}. This will issue a warning '... might not work in parallel mode', but is perfectly fine for non-parallel mode. 
 #'
 #' @examples
-#'    ## Load the best results obtained in a prior tuning for the configuration "sonar_04.conf" with tuning method "spot".
-#'    ## The result envT from a prior run of tdmBigLoop with this .conf is read from demo02sonar/demoSonar.RData.
-#'    ## Run task main_sonar again with these best parameters, using the default settings from tdmDefaultsFill 
-#'    ## umode="RSUB", tdm$nrun=5  and tdm$tstFrac=0.2.
+#'    ## Load the best results obtained in a prior tuning for the configuration 
+#'    ## "sonar_04.conf" with tuning method "spot". The result envT from a prior  
+#'    ## run of tdmBigLoop with this .conf is read from demo02sonar/demoSonar.RData.
+#'    ## Run task main_sonar again with these best parameters, using the default 
+#'    ## settings from tdmDefaultsFill: umode="RSUB", tdm$nrun=5  and tdm$TST.testFrac=0.2.
 #'    oldwd <- getwd();          
-#'    ## The best results are read from demo02sonar/demoSonar.RData relative to the TDMR package directory.
+#'    ## The best results are read from demo02sonar/demoSonar.RData relative to 
+#'    ## the TDMR package directory.
 #'    setwd(paste(.find.package("TDMR"), "demo02sonar",sep="/"));
 #'    load("demoSonar.RData");
 #'    source("main_sonar.r");
@@ -70,7 +70,7 @@
 #    ## (This example is not run automatically, because sonar_04.bst is not in the package distribution) 
 #    setwd(paste(.find.package("TDMR"), "demo02sonar",sep="/"));
 #    envT <- new.env();
-#    tdm <- list(mainFunction="main_sonar", tuneMethod="spot");
+#    tdm <- list(mainFunc="main_sonar", tuneMethod="spot");
 #    source("main_sonar.r");
 #    finals <- unbiasedRun("sonar_04.conf",envT,tdm=tdm);
 #    print(finals);
@@ -78,7 +78,7 @@
 #    }
 #'
 #' @seealso   \code{\link{tdmBigLoop}}
-#' @author Wolfgang Konen, FHK, Sep'2010 - Dec'2011
+#' @author Wolfgang Konen, FHK, 2010 - 2013
 #' @export
 #
 ######################################################################################
@@ -89,7 +89,7 @@ unbiasedRun <- function(confFile,envT,dataObj=NULL,finals=NULL,umode="RSUB",with
     if (is.null(envT$nExp)) envT$nExp <- 1;
     if (is.null(tdm$map)) tdm <- tdmMapDesLoad(tdm); 
     #if (is.null(envT$map)) tdmMapDesLoad(envT,tdm); 
-    
+
     if (is.null(envT$spotConfig$opts)) {
       if (is.null(envT$sCList[[1]]$opts)) {
         pdFile = envT$spotConfig$io.apdFileName;
@@ -118,42 +118,47 @@ unbiasedRun <- function(confFile,envT,dataObj=NULL,finals=NULL,umode="RSUB",with
       tdm$tstCol=dataObj$TST.COL;  # needed for tdmMapOpts
       #dset <- dataObj$dset;    
       dset <- dsetTrnVa(dataObj);
-      tset <- dsetTest(dataObj);
+      tset <- dsetTest(dataObj);       # returns NULL in case of (tdm$umode %in% c("RSUB","CV")
     } else {
+      # this branch is deprecated 
+      if (is.null(dataObj))
+        cat("NOTE: setting opts$READ.INI==FALSE together with argument dataObj==NULL in unbiasedRun.r is deprecated. Consider opts$READ.INI=TRUE.");  
       dset <- NULL;
       tset <- NULL;
     }
     # set certain elements of opts which control selection of training & test data 
     # (depending on switch tdm$umode, see tdmMapDesign.r)
-  	opts <- tdmMapOpts(umode,opts,tdm);           # sets opts$NRUN = tdm$nrun
+  	tdmOpts <- tdmMapOpts(umode,opts,tdm);           # sets opts$NRUN = tdm$nrun
 
-    # deprecated, only for downward compatibility. For actual runs, a simple
+    # tdmGetObj is deprecated, only for downward compatibility. For actual runs, a simple
     #   bst <- envT$bst
     #   res <- envT$res
     # would be sufficient.
+#'     If envT$bst or envT$res is NULL, try to read it from the file (the filename is
+#'     inferred envT$spotConfig. If this is NULL, it is constructed from confFile). 
+#'     We try to find the files for envT$bst or envT$res in dir envT$theTuner).
     bst <- tdmGetObj(envT$bst,envT$spotConfig$io.bstFileName,envT$theTuner,tdm);
     res <- tdmGetObj(envT$res,envT$spotConfig$io.resFileName,envT$theTuner,tdm);
 
     k <- nrow(bst);       # last line has the best solution
     #bst <- tdmMapCutoff(bst,k,envT$spotConfig);  # enforce CUTOFF parameter constraint if CUTOFF2[,3,4] appears in .des-file
-  	opts <- tdmMapDesApply(bst,opts,k,envT$spotConfig,tdm);
-  	if (is.null(opts$fileMode)) opts$fileMode=TRUE;    # might be necessary for older opts from file
+  	tdmOpts <- tdmMapDesApply(bst,tdmOpts,k,envT$spotConfig,tdm);
+  	if (is.null(tdmOpts$fileMode)) tdmOpts$fileMode=TRUE;    # might be necessary for older opts from file
 
   	cat("Best solution:\n"); print(bst[k,]);
   	#
-  	# now opts has the best solution obtained in a prior tuning, and it has 
+  	# now tdmOpts has the best solution obtained in a prior tuning, and it has 
   	# training and test set configured according to the current setting of umode
 
 		conf <- as.numeric(bst$CONFIG[k])
 		cat(sprintf("Best Config: %5d\n\n",conf))
-
     #
-    # run new model trainings for the configuration stored in opts (tdm$nrun trainings & evaluations):
+    # run new model trainings for the configuration stored in tdmOpts (tdm$nrun trainings & evaluations):
     #
 		oldwd = getwd();                                               #
     if (!is.null(tdm$mainFile)) setwd(dirname(tdm$mainFile));      # save & change working dir
-		result = NULL;         
-  	mainCommand <- paste("result <- ", tdm$mainFunc,"(opts,dset=dset,tset=tset)",sep=" ");
+		result = NULL;        
+  	mainCommand <- paste("result <- ", tdm$mainFunc,"(tdmOpts,dset=dset,tset=tset)",sep=" ");
     eval(parse(text=mainCommand));                  # execute the command given in text string mainCommand
     if (!is.list(result)) stop("tdm$mainFunc did not return a list 'result'");
 #--- only for testing parallel execution: comment 2 lines above out and uncomment 2 lines below:
@@ -173,26 +178,26 @@ unbiasedRun <- function(confFile,envT,dataObj=NULL,finals=NULL,umode="RSUB",with
         names(finals) <- c(namFinals,pNames);     #
       } 
       finals <- cbind(finals
-                     , NRUN=opts$NRUN
+                     , NRUN=tdmOpts$NRUN
                      , NEVAL=nrow(res)
                       );                      
       sgn <- ifelse(class(envT$result)[1]=="TDMregressor",+1,-1);   # minus sign only for classification
       add.finals        <- data.frame(sgn*as.numeric(bst[k,1]),sgn*mean(res$Y));
-      names(add.finals) <- paste(opts$rgain.string,c(".bst",".avg"),sep="");
+      names(add.finals) <- paste(tdmOpts$rgain.string,c(".bst",".avg"),sep="");
       finals <- cbind(finals,add.finals);
       #
       # add once the results on the training set from unbiased runs:                      
-      suf = ifelse(opts$MOD.method %in% c("RF","MC.RF"),".OOB",".TRN");
+      suf = ifelse(tdmOpts$MOD.method %in% c("RF","MC.RF"),".OOB",".TRN");
       add.finals <-  data.frame( mean(result$R_train)
                                , sd(result$R_train)
                                );
-      names(add.finals) <- paste(c(opts$rgain.string,"sdR"),suf,sep="");
+      names(add.finals) <- paste(c(tdmOpts$rgain.string,"sdR"),suf,sep="");
       finals <- cbind(finals,add.finals);
     } # if(is.null(finals))
       
     # add results on test set from unbiased runs (two columns for each value of umode):
     add.finals <- data.frame(mean(result$R_test),sd(result$R_test));
-    names(add.finals) <-  paste(c(opts$rgain.string,"sdR"),umode,sep=".");
+    names(add.finals) <-  paste(c(tdmOpts$rgain.string,"sdR"),umode,sep=".");
     finals <- cbind(finals,add.finals);
 
     finals;
