@@ -13,25 +13,27 @@ setwd(path);
 source("main_sonar.r");    # in working dir
 
 ## preliminary settings for TDMR
-tdm <- list( mainFunction="main_sonar"
-            , umode=c("RSUB")           # ["CV" | "RSUB" | "TST"]
+tdm <- list( mainFunc="main_sonar"
+            , runList = "sonar_05.conf"
+            , umode=c("RSUB")           # ["CV" | "RSUB" | "TST" | "SP_T" ]
             , tuneMethod = c("spot","lhd")
             , filenameEnvT="demo03.RData"   # file to save environment envT (in working dir)
-            , finalFile="sonar.fin"
             , nrun=5, nfold=2         # repeats and CV-folds for the unbiased runs
-            , withParams=TRUE         # list the columns with tuned parameter in final results 
             , nExperim=2
-            , parallelCPUs=2
+            , parallelCPUs=1
+            , parallelFuncs="readCmdSonar"  # fct's to export in addition to tdm$mainFunc in case parallelCPUs>1
             , optsVerbosity = 0       # the verbosity for the unbiased runs
             );
-## fill in other defaults for list tdm            
-tdm <- tdmDefaultsFill(tdm);      
-## sonar_05.conf has the settings for the tuning process (e.g. "auto.loop.steps"=number of SPOT generations       
-## "auto.loop.evals"=budget of model building runs and io.roiFileName = "sonar_04.roi").
-## runList could contain other files as well (e.g. c("sonar_01.conf","sonar_02.conf","sonar_03.conf")), if desired.
-tdm$runList = c("sonar_05.conf");     
-## spotStep can be either "auto" (do automatic tuning) or "rep" (make a visual report of best results)
-spotStep = "auto";
+## tdm$runList="sonar_05.conf" has the settings for the tuning process (e.g. 
+##    - "auto.loop.steps"=number of SPOT generations       
+##    - "auto.loop.evals"=budget of model building runs and 
+##    - io.roiFileName = "sonar_05.roi"
+## ). tdm$runList could contain other files as well (e.g. 
+##    c("sonar_01.conf","sonar_02.conf","sonar_03.conf")
+## ), if desired.
+
+spotStep = "auto";    ## spotStep can be either "auto" (do automatic tuning) or 
+            ## "rep" (make a visual report and an unbiased run on best results)
 
 ## construct an initial environment envT from the given TDMR settings in tdm
 ## (this contains also the fill-in of other defaults for tdm via
@@ -40,7 +42,8 @@ spotStep = "auto";
 envT <- tdmEnvTMakeNew(tdm);
 
 ## the call to tdmBigLoop will start the whole TDMR process:
-## - for each file in tdm$runList a complete DM tuning is started with each tuning method tdm$tuneMethod  (if spotStep=="auto")
+## - for each file in tdm$runList a complete DM tuning is started with each tuning 
+##   method tdm$tuneMethod  (if spotStep=="auto")
 ## - the best result from tuning is fed into an unbiased model build and evaluation run 
 ## - results are printed and returned in envT$theFinals 
 ## - more detailed results are in other elements of environment envT
@@ -49,20 +52,7 @@ envT <- tdmEnvTMakeNew(tdm);
 ##      b) the sensitivity plot for each parameter in the vicinity of the best solution found 
 envT <- tdmBigLoop(envT,spotStep);
 
-#---- deprecated ----
-## the call to tdmCompleteEval will start the whole TDMR process:
-## - for each file in runList a complete DM tuning is started with each tuning method
-## - the best result from tuning are fed into an unbiased model building and evaluation run 
-## - results are printed and returned in envT$theFinals 
-## - more detailed results are in other elements of envT
-## - two plots: 
-##      a) the progression of the response variable Y and the parameter variables during tuning
-##      b) the sensitivity plot for each parameter in the vicinity of the best solution found 
-#envT <- tdmCompleteEval(runList,NULL,spotStep,tdm);
-#---- deprecated ----
-
-## restore old working directory
-setwd(oldwd);
+setwd(oldwd);               # restore old working directory
 
 ## the resulting tuning surface (the metamodel) can be inspected interactively with
 ##      load(paste(path,tdm$filenameEnvT,sep="/");     
